
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "cell.h"
#include "rif.h"
#include "sif.h"

extern double	Convert(/* double value */);

static void	ConvertValue(path1,base1,step1,
			     path2,base2,step2,pathO,baseO,stepO,BPS)
char		*path1,*path2,*pathO;
double		base1,step1,base2,step2,baseO,stepO;
int		BPS;
{
	int	Nx,Ny,x,y;
	Cell	**cell1,**cell2,**cell,cell0=(Cell)((1<<BPS)-1);
	double	value1,value2,value,value0=baseO+stepO*(double)cell0;

	ReadImageFile(path1,&Nx,&Ny,NULL,&cell1,NULL);
	ReadImageFile(path2,&x ,&y ,NULL,&cell2,NULL); cell=cell2;

	if (Nx!=x || Ny!=y) {
	    fprintf(stderr,"%s / %s : image size not match.\n",path1,path2);
	    exit(1);
	}

	for (y=0; y<Ny; y++)
	for (x=0; x<Nx; x++) {
	    value1=Convert(base1+step1*(double)cell1[y][x]);
	    value2=Convert(base2+step2*(double)cell2[y][x]);
#ifdef	ADD
	    value=value1+value2;
#endif
#ifdef	SUB
	    value=value1-value2;
#endif
#ifdef	MUL
	    value=value1*value2;
#endif
#ifdef	DIV
value=(value2==0.0)?(value1==0.0)?0.0:(value1<0.0)?baseO:value0:value1/value2;
#endif
	    if (value<baseO)
		cell[y][x]=0;
	    else if (value>value0)
		cell[y][x]=cell0;
	    else
		cell[y][x]=(Cell)floor((value-baseO)/stepO+0.5);
	}
	StoreImageFile(pathO,Nx,Ny,BPS,cell,NULL);

	for (y=Ny-1; y>=0; y--) free(cell2[y]); free(cell2);
	for (y=Ny-1; y>=0; y--) free(cell1[y]); free(cell1);
}

#define PATH_LEN	256

int	main(argc,argv)
int	argc;
char	**argv;
{
	char	*ptr,*str,path1[PATH_LEN],path2[PATH_LEN],pathO[PATH_LEN];
	double	baseO,stepO,base1,step1,base2,step2;
	int	BPS;

	if (argc!=4 && argc!=8 && argc!=11) {
	    for (ptr=str=argv[0]; *ptr!='\0'; ptr++) if (*ptr=='/') str=ptr+1;

	    fprintf(stderr,"usage : %s p1 b1 s1 p2 b2 s2 pO bO sO BPS\n"
			   "usage : %s b1 s1 b2 s2 bO sO BPS\n"
			   "usage : %s bO sO BPS\n",str,str,str); exit(1);
	}
	baseO=atof(argv[argc-3]);

	if ((stepO=atof(argv[argc-2]))<=0.0) {
	    fputs("bad step value.\n",stderr); exit(1);
	}
	if ((BPS=atoi(argv[argc-1]))<=0 || BPS>8*sizeof(Cell)) {
	    fputs("bad BPS (Bits Per Sample).\n",stderr); exit(1);
	}
	if (argc==11)
	    ConvertValue(argv[1],atof(argv[2]),atof(argv[3]),
			 argv[4],atof(argv[5]),atof(argv[6]),
			 argv[7],baseO,stepO,BPS);
	else if (argc==8) {
	    base1=atof(argv[1]);
	    step1=atof(argv[2]);
	    base2=atof(argv[3]);
	    step2=atof(argv[4]);

	    while (scanf("%s %s %s",path1,path2,pathO)==3)
		ConvertValue(path1,base1,step1,
			     path2,base2,step2,pathO,baseO,stepO,BPS);
	}
	else
	    while (scanf("%s %lf %lf %s %lf %lf %s",
			 path1,&base1,&step1,path2,&base2,&step2,pathO)==7)
		ConvertValue(path1,base1,step1,
			     path2,base2,step2,pathO,baseO,stepO,BPS);

	return 0;
}
